package java.security;

import java.security.spec.AlgorithmParameterSpec;
import sun.security.jca.JCAUtil;
import java.util.Iterator;
import sun.security.jca.GetInstance;
import sun.security.util.Debug;

public abstract class KeyPairGenerator extends KeyPairGeneratorSpi
{
    private static final Debug pdebug;
    private static final boolean skipDebug;
    private final String algorithm;
    Provider provider;
    
    protected KeyPairGenerator(String algorithm) {
        this.algorithm = algorithm;
    }
    
    public String getAlgorithm() {
        return this.algorithm;
    }
    
    private static KeyPairGenerator getInstance(GetInstance.Instance instance, String str) {
        KeyPairGenerator keyPairGenerator;
        if ((instance.impl instanceof KeyPairGenerator)) {
            keyPairGenerator = (KeyPairGenerator)instance.impl;
        }
        else {
            keyPairGenerator = new KeyPairGenerator.Delegate((KeyPairGeneratorSpi)instance.impl, str);
        }
        keyPairGenerator.provider = instance.provider;
        if (!KeyPairGenerator.skipDebug && KeyPairGenerator.pdebug != null) {
            KeyPairGenerator.pdebug.println("KeyPairGenerator." + str + " algorithm from: " + keyPairGenerator.provider.getName());
            return keyPairGenerator;
        }
        return keyPairGenerator;
    }
    
    public static KeyPairGenerator getInstance(String s) throws NoSuchAlgorithmException {
        Iterator<Provider.Service> iterator = GetInstance.getServices("KeyPairGenerator", s).iterator();
        if (!iterator.hasNext()) {
            throw new NoSuchAlgorithmException(s + " KeyPairGenerator not available");
        }
        NoSuchAlgorithmException ex = null;
        while (true) {
            Provider.Service s2 = (Provider.Service)iterator.next();
            try {
                GetInstance.Instance instance = GetInstance.getInstance(s2, KeyPairGeneratorSpi.class);
                if ((instance.impl instanceof KeyPairGenerator)) {
                    return getInstance(instance, s);
                }
                return new KeyPairGenerator.Delegate(instance, iterator, s);
            }
            catch (NoSuchAlgorithmException ex2) {
                if (ex == null) {
                    ex = ex2;
                }
                if (iterator.hasNext()) {
                    continue;
                }
                throw ex;
            }
        }
    }
    
    public static KeyPairGenerator getInstance(String algorithm, String provider) throws NoSuchAlgorithmException, NoSuchProviderException {
        return getInstance(GetInstance.getInstance("KeyPairGenerator", KeyPairGeneratorSpi.class, algorithm, provider), algorithm);
    }
    
    public static KeyPairGenerator getInstance(String algorithm, Provider provider) throws NoSuchAlgorithmException {
        return getInstance(GetInstance.getInstance("KeyPairGenerator", KeyPairGeneratorSpi.class, algorithm, provider), algorithm);
    }
    
    public final Provider getProvider() {
        this.disableFailover();
        return this.provider;
    }
    
    void disableFailover() {
    }
    
    public void initialize(int n) {
        this.initialize(n, JCAUtil.getSecureRandom());
    }
    
    @Override
    public void initialize(int n, SecureRandom secureRandom) {
    }
    
    public void initialize(AlgorithmParameterSpec algorithmParameterSpec) throws InvalidAlgorithmParameterException {
        this.initialize(algorithmParameterSpec, JCAUtil.getSecureRandom());
    }
    
    @Override
    public void initialize(AlgorithmParameterSpec algorithmParameterSpec, SecureRandom secureRandom) throws InvalidAlgorithmParameterException {
    }
    
    public final KeyPair genKeyPair() {
        return this.generateKeyPair();
    }
    
    @Override
    public KeyPair generateKeyPair() {
        return null;
    }
    
    static {
        pdebug = Debug.getInstance("provider", "Provider");
        skipDebug = (Debug.isOn("engine=") && !Debug.isOn("keypairgenerator"));
    }
    
    private static final class Delegate extends KeyPairGenerator
    {
        private volatile KeyPairGeneratorSpi spi;
        private final Object lock;
        private Iterator<Provider.Service> serviceIterator;
        private static final int I_NONE = 1;
        private static final int I_SIZE = 2;
        private static final int I_PARAMS = 3;
        private int initType;
        private int initKeySize;
        private AlgorithmParameterSpec initParams;
        private SecureRandom initRandom;
        
        Delegate(KeyPairGeneratorSpi spi, String s) {
            super(s);
            this.lock = new Object();
            this.spi = spi;
        }
        
        Delegate(GetInstance.Instance instance, Iterator<Provider.Service> serviceIterator, String str) {
            super(str);
            this.lock = new Object();
            this.spi = (KeyPairGeneratorSpi)instance.impl;
            this.provider = instance.provider;
            this.serviceIterator = serviceIterator;
            this.initType = 1;
            if (!KeyPairGenerator.skipDebug && KeyPairGenerator.pdebug != null) {
                KeyPairGenerator.pdebug.println("KeyPairGenerator." + str + " algorithm from: " + this.provider.getName());
            }
        }
        
        private KeyPairGeneratorSpi nextSpi(KeyPairGeneratorSpi keyPairGeneratorSpi, boolean b) {
            synchronized (this.lock) {
                if (keyPairGeneratorSpi != null && keyPairGeneratorSpi != this.spi) {
                    return this.spi;
                }
                if (this.serviceIterator == null) {
                    return null;
                }
                while (this.serviceIterator.hasNext()) {
                    Provider.Service service = (Provider.Service)this.serviceIterator.next();
                    try {
                        Object instance = service.newInstance(null);
                        if (!(instance instanceof KeyPairGeneratorSpi)) {
                            continue;
                        }
                        if ((instance instanceof KeyPairGenerator)) {
                            continue;
                        }
                        KeyPairGeneratorSpi spi = (KeyPairGeneratorSpi)instance;
                        if (b) {
                            if (this.initType == 2) {
                                spi.initialize(this.initKeySize, this.initRandom);
                            }
                            else if (this.initType == 3) {
                                spi.initialize(this.initParams, this.initRandom);
                            }
                            else if (this.initType != 1) {
                                throw new AssertionError("KeyPairGenerator initType: " + this.initType);
                            }
                        }
                        this.provider = service.getProvider();
                        this.spi = spi;
                        return spi;
                    }
                    catch (Exception ex) {}
                }
                this.disableFailover();
                return null;
            }
        }
        
        @Override
        void disableFailover() {
            this.serviceIterator = null;
            this.initType = 0;
            this.initParams = null;
            this.initRandom = null;
        }
        
        @Override
        public void initialize(int initKeySize, SecureRandom initRandom) {
            if (this.serviceIterator == null) {
                this.spi.initialize(initKeySize, initRandom);
                return;
            }
            RuntimeException ex = null;
            KeyPairGeneratorSpi keyPairGeneratorSpi = this.spi;
            Label_0025: {
                try {
                    keyPairGeneratorSpi.initialize(initKeySize, initRandom);
                    this.initType = 2;
                    this.initKeySize = initKeySize;
                    this.initParams = null;
                    this.initRandom = initRandom;
                    return;
                }
                catch (RuntimeException ex2) {
                    if (ex == null) {
                        ex = ex2;
                    }
                    keyPairGeneratorSpi = this.nextSpi(keyPairGeneratorSpi, false);
                    if (keyPairGeneratorSpi != null) {
                        continue;
                    }
                    throw ex;
                }
            }
        }
        
        @Override
        public void initialize(AlgorithmParameterSpec initParams, SecureRandom initRandom) throws InvalidAlgorithmParameterException {
            if (this.serviceIterator == null) {
                this.spi.initialize(initParams, initRandom);
                return;
            }
            Exception ex = null;
            KeyPairGeneratorSpi keyPairGeneratorSpi = this.spi;
            Label_0025: {
                try {
                    keyPairGeneratorSpi.initialize(initParams, initRandom);
                    this.initType = 3;
                    this.initKeySize = 0;
                    this.initParams = initParams;
                    this.initRandom = initRandom;
                    return;
                }
                catch (Exception ex2) {
                    if (ex == null) {
                        ex = ex2;
                    }
                    keyPairGeneratorSpi = this.nextSpi(keyPairGeneratorSpi, false);
                    if (keyPairGeneratorSpi != null) {
                        continue;
                    }
                    if ((ex instanceof RuntimeException)) {
                        throw (RuntimeException)ex;
                    }
                    throw (InvalidAlgorithmParameterException)ex;
                }
            }
        }
        
        @Override
        public KeyPair generateKeyPair() {
            if (this.serviceIterator == null) {
                return this.spi.generateKeyPair();
            }
            RuntimeException ex = null;
            KeyPairGeneratorSpi keyPairGeneratorSpi = this.spi;
            try {
                return keyPairGeneratorSpi.generateKeyPair();
            }
            catch (RuntimeException ex2) {
                if (ex == null) {
                    ex = ex2;
                }
                keyPairGeneratorSpi = this.nextSpi(keyPairGeneratorSpi, true);
                if (keyPairGeneratorSpi != null) {
                    return keyPairGeneratorSpi.generateKeyPair();
                }
                throw ex;
            }
        }
    }
}
